%=========================================================================%
% Positive feedback loop switch.
%
% Deterministic Fractional Occupancy Model
% iGEM 2012 Team Slovenia - http://2012.igem.org/Team:Slovenia
%=========================================================================%



plot_BFP = 1;          % 1 = we're plotting BFP state, 0 = mCitrine
top_view = 0;          % Plot side-view 3D or top-view 2D?



TIME = 250;            % Simulation duration.
SamplesPerStep = 200;
SAMPLES = TIME*SamplesPerStep;
dt = 1/SamplesPerStep;  % Time step.


% Initial values/concentrations.
BFP = 0;
MCitrin = 0;
TAL97_KRAB = 0;
TAL57_KRAB = 0;
TAL97_VP16 = 0;
TAL57_VP16 = 0;
PIPKRAB = 0;
EKRAB = 0;

Signal1 = 0;
Signal2 = 0;



% Fractional occupancy parameters - association constants and exponents.
k1 = 1;         % TAL57_VP16 - construct 1
k2 = 1;         % TAL57_KRAB - construct 1
k3 = 1;         % TAL97_VP16 - construct 2
k4 = 1;         % TAL97_KRAB - construct 2
k5 = 1;         % PIPKRAB
k6 = 1;         % EKRAB

n1 = 1.5;         % TAL57_VP16 - construct 1
n2 = 1.5;         % TAL57_KRAB - construct 1
n3 = 1.5;         % TAL97_VP16 - construct 2
n4 = 1.5;         % TAL97_KRAB - construct 2
n5 = 1.5;         % PIPKRAB
n6 = 1.5;         % EKRAB

Ka = 3;     % Amount of activator required for 50% activation (minimal promoter).
Kr = 1;     % Amount of repressor required for 50% repression (constitutive promoter).


Ka = 3;     % Amount of activator required for 50% activation (minimal promoter).
Kr = 1;     % Amount of repressor required for 50% repression (constitutive promoter).



% Production & degradation parameters.
k_rep1 = 10;                % BFP production rate (active promoter).
kb_rep1 = 0.00;             % BFP basal rate (leaking - inactive promoter).
deg_rep1 = 0.1;             % BFP degradation rate.

k_rep2 = 10;                % MCitrin production rate (active promoter).
kb_rep2 = 0.00;             % MCitrin basal rate (leaking - inactive promoter).
deg_rep2 = 0.1;             % MCitrin degradation rate.
   
k1_TAL97_KRAB = 10;         % TAL97_KRAB production rate from construct 1.
kb1_TAL97_KRAB = 0.25;      % TAL97_KRAB basal rate from construct 1.
k3_TAL97_KRAB = 10;         % TAL97_KRAB production rate from construct 3.
kb3_TAL97_KRAB = 0.25;      % TAL97_KRAB basal rate from construct 3.
deg_TAL97_KRAB = 0.1;       % TAL97_KRAB degradation rate.

k2_TAL57_KRAB = 10;         % TAL57_KRAB production rate from construct 2.
kb2_TAL57_KRAB = 0.25;      % TAL57_KRAB basal rate from construct 2.
k4_TAL57_KRAB = 10;         % TAL57_KRAB production rate from construct 4.
kb4_TAL57_KRAB = 0.25;      % TAL57_KRAB basal rate from construct 4.
deg_TAL57_KRAB = 0.1;       % TAL57_KRAB degradation rate.

k1_TAL57_VP16 = 10;         % TAL57_VP16 production rate from construct 1.
kb1_TAL57_VP16 = 0.25;      % TAL57_VP16 basal rate from construct 1.
k3_TAL57_VP16 = 10;         % TAL57_VP16 production rate from construct 3.
kb3_TAL57_VP16 = 0.25;      % TAL57_VP16 basal rate from construct 3.
deg_TAL57_VP16 = 0.1;       % TAL57_VP16 degradation rate.

k2_TAL97_VP16 = 10;         % TAL97_VP16 production rate from construct 2.
kb2_TAL97_VP16 = 0.25;      % TAL97_VP16 basal rate from construct 2.
k4_TAL97_VP16 = 10;         % TAL97_VP16 production rate from construct 4.
kb4_TAL97_VP16 = 0.25;      % TAL97_VP16 basal rate from construct 4.
deg_TAL97_VP16 = 0.1;       % TAL97_VP16 degradation rate.

k_PIPKRAB = 10;             % PIPKRAB production rate.
deg_PIPKRAB = 0.1;          % PIPKRAB degradation rate.

k_EKRAB = 10;              % EKRAB production rate.
deg_EKRAB = 0.1;           % EKRAB degradation rate.


deg_PIPKRAB_fast = 100;     % PIPKRAB degradation rate when signal1 is present.
deg_PIPKRAB_normal = 0.1;   % PIPKRAB degradation rate when signal1 is absent (normal degradation rate).

deg_EKRAB_fast = 100;      % EKRAB degradation rate when signal2 is present.
deg_EKRAB_normal = 0.1;    % EKRAB degradation rate when signal2 is absent (normal degradation rate).





% Parameter space to check
%========== Exponents (functional cooperativity) test. ==========%.
param_start = 1;           % Starting value.
param_end = 4;             % Ending value.
param_step = 0.5;          % Step.

        
% What are we plotting (at each time for each parameter value)
myz = zeros(0:dt:TIME, size(param_start:param_step:param_end, 2));


% Check parameter space.
i=1;
for p1=param_start:param_step:param_end
    
    % Initial values/concentrations.
    BFP = 0;
    MCitrin = 0;
    TAL97_KRAB = 0;
    TAL57_KRAB = 0;
    TAL97_VP16 = 0;
    TAL57_VP16 = 0;
    PIPKRAB = 0;
    EKRAB = 0;

    Signal1 = 0;
    Signal2 = 0;

    %========== Exponents (cooperativity) test. ==========%
    n1 = p1;
    n2 = p1;
    n3 = p1;
    n4 = p1;
    n5 = p1;
    n6 = p1;
    
                  
    %========== SIMULATION ==========%
    j=1;
    for t=0:dt:(TIME-1)    
        if t==0
            Signal1 = 1;
        end
        if t==50            
            Signal1 = 0;
        end

        if t==100
            Signal2 = 1;
        end
        if t==150
            Signal2 = 0;
        end
             
        % Calculate fractional occupancies    
        f1 = (k1*TAL57_VP16^n1) / (Ka + k1*TAL57_VP16^n1 + k2*TAL57_KRAB^n2);
        f2 = (k3*TAL97_VP16^n3) / (Ka + k3*TAL97_VP16^n3 + k4*TAL97_KRAB^n4);
        f3 = Kr/(Kr+k5*PIPKRAB^n5);               
        f4 = Kr/(Kr+k6*EKRAB^n6); 


        % Signal1 present -> PIPKRAB cannot bind (cannot repress construct 3).
        % This is modeled as fast PIPKRAB degradation in the presence of signal1.
        if Signal1==1
            deg_PIPKRAB = deg_PIPKRAB_fast;
        else
            deg_PIPKRAB = deg_PIPKRAB_normal;
        end

        % Similar for EKRAB (promoter 4 repression).
        if Signal2==1
            deg_EKRAB = deg_EKRAB_fast;
        else
            deg_EKRAB = deg_EKRAB_normal;
        end


        % ODEs.
        % Form: dProtein/dt = production_rate * P(active promoter) + basal_rate*P(inactive promoter) - degradation_rate*Protein
        dBFP = dt * (k_rep1*f1 + kb_rep1*(1-f1) - deg_rep1*BFP);
        dMCitrin = dt * (k_rep2*f2 + kb_rep2*(1-f2) - deg_rep2*MCitrin);
        dTAL97_KRAB = dt * (k1_TAL97_KRAB*f1 + kb1_TAL97_KRAB*(1-f1) + k3_TAL97_KRAB*f3 + kb3_TAL97_KRAB*(1-f3) - deg_TAL97_KRAB*TAL97_KRAB);
        dTAL57_KRAB = dt * (k2_TAL57_KRAB*f2 + kb2_TAL57_KRAB*(1-f2) + k4_TAL57_KRAB*f4 + kb4_TAL57_KRAB*(1-f4) - deg_TAL57_KRAB*TAL57_KRAB);    
        dTAL97_VP16 = dt * (k2_TAL97_VP16*f2 + kb2_TAL97_VP16*(1-f2) + k4_TAL97_VP16*f4 + kb4_TAL97_VP16*(1-f4) - deg_TAL97_VP16*TAL97_VP16);
        dTAL57_VP16 = dt * (k1_TAL57_VP16*f1 + kb1_TAL57_VP16*(1-f1) + k3_TAL57_VP16*f3 + kb3_TAL57_VP16*(1-f3) - deg_TAL57_VP16*TAL57_VP16);    
        dPIPKRAB = dt * (k_PIPKRAB - deg_PIPKRAB*PIPKRAB);
        dEKRAB = dt * (k_EKRAB - deg_EKRAB*EKRAB);   

        % Update values.
        BFP = BFP + dBFP;
        MCitrin = MCitrin + dMCitrin;
        TAL97_KRAB = TAL97_KRAB + dTAL97_KRAB;
        TAL97_VP16 = TAL97_VP16 + dTAL97_VP16;
        TAL57_KRAB = TAL57_KRAB + dTAL57_KRAB;
        TAL57_VP16 = TAL57_VP16 + dTAL57_VP16;
        PIPKRAB = PIPKRAB + dPIPKRAB;
        EKRAB = EKRAB + dEKRAB;        

        if (PIPKRAB < 0) PIPKRAB = 0; end
        if (EKRAB < 0) EKRAB = 0; end
        
        myz(j,i) = BFP;
        
        if (plot_BFP == 0)
            myz(j,i) = MCitrin;
        end
        
        j=j+1;
    end
    
    i=i+1;
end 



if (plot_BFP == 1)
    colormap winter;
else     
    colormap summer;
end

set(gcf,'Color',[1 1 1]);

[X Y] = meshgrid(param_start:param_step:param_end, 0:dt:(TIME-1));
M=surf(X,Y, myz, 'EdgeColor','none', 'FaceColor', 'interp', 'FaceLighting','Phong', 'FaceAlpha', 0.8);

if (top_view == 0)
    set(M,'FaceLighting','phong','FaceColor','interp','AmbientStrength',0.8)
    light('Position',[100 200 400],'Style','infinite');
end

screensize = get(0, 'ScreenSize');
set(gcf,'Position',[screensize(3)/2 - 350, screensize(4)/2 - 160, 1000, 680]);

xlabel('TF cooperativity');

ylabel('Time');

if (plot_BFP == 1)
    zlabel('BFP');
else
    zlabel('mCitrine');
end


if (plot_BFP == 1)
    export_fig '.\PositiveLoopSwitch_parameterTest1_BFP_b.png'
else
    export_fig '.\PositiveLoopSwitch_parameterTest1_Cit_b.png'
end


% Top-view
if (top_view == 1)
    view(0,90)
    colorbar
    
    if (plot_BFP == 1)
        export_fig '.\PositiveLoopSwitch_parameterTest1_BFP_top_b.png'
    else
        export_fig '.\PositiveLoopSwitch_parameterTest1_Cit_top_b.png'
    end
end   


